﻿using System;
using ServiceStack.ServiceInterface;

namespace SPS.ServiceCore.API.ServiceInterface
{
    public abstract class SPSServiceBase<TRequest> : RestServiceBase<TRequest>
    {
		public IAuthProvider AuthProvider { get; set; }

		public override void OnBeforeExecute(TRequest request)
		{
			Validate(request);
		}

		private void Validate(TRequest request)
		{
			var requiresAuth = request as IRequiresAuthentication;
			if (requiresAuth == null) return;

			var validatedSession = this.AuthProvider.GetValidSession(requiresAuth.SessionId);

			var attrs = request.GetType().GetCustomAttributes(typeof (CustomAuthAttribute), true);
			if (attrs.Length > 0)
			{
				var customAuth = (CustomAuthAttribute)attrs[0];

				if (validatedSession.ClientType != customAuth.ClientType)
					throw new UnauthorizedAccessException("Unauthorized. Requires ClientType: " + customAuth.ClientType);
			}
        }
    }
}
