using System;
using System.Collections.Generic;

namespace SPS.ServiceCore.API.ServiceInterface
{
	public class InMemoryAuthProvider : IAuthProvider
	{
		readonly Dictionary<Guid, AuthSession> authSessions = new Dictionary<Guid, AuthSession>();

		public AuthSession CreateSession(string userName, string authKey)
		{
			var authSession = new AuthSession
			{
				SessionId = Guid.NewGuid(),
				UserName = userName,
				ExpiresAt = DateTime.Now.AddDays(14),
			};

			//Pretend you got ClientType, Roles, what else you need, from a DB
			if (userName == "HazSuperUser")
			{
				authSession.ClientType = "SuperUser";
			}

			authSessions[authSession.SessionId] = authSession;

			return authSession;
		}

		public AuthSession GetValidSession(Guid sessionId)
		{
			if (sessionId == default(Guid))
				throw new UnauthorizedAccessException("SessionId Required");

			AuthSession authSession;
			if (authSessions.TryGetValue(sessionId, out authSession))
			{
				if (DateTime.Now > authSession.ExpiresAt)
					throw new UnauthorizedAccessException("Expired at: " + authSession.ExpiresAt);

				return authSession;
			}

			throw new UnauthorizedAccessException("Session does not exist: " + sessionId);
		}
	}
}