﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using SPS.ServiceCore.API.ServiceModel.Operations;
using ServiceStack.Service;
using ServiceStack.ServiceClient.Web;

namespace SPS.ServiceCore.API.Test
{
	[TestClass]
	public class AuthenticationTests
	{
		private const string HasSuccess = "Haz Success!";

		private static IRestClient restClient;

		[ClassInitialize]
		public static void OnBeforeTestFixture(TestContext ctx)
		{
			restClient = (IRestClient)new JsonServiceClient("http://localhost/SPS.ServiceCore.API");
			//restClient = (IRestClient)new JsonServiceClient("http://localhost:5000");
		}

		[TestMethod]
		public void Can_Access_PublicService()
		{
			var response = restClient.Get<PublicResponse>("/public");

			Assert.AreEqual(response.Result, HasSuccess);
		}

		[TestMethod]
		public void Can_NOT_Access_ProtectedService()
		{
			try
			{
				var response = restClient.Get<ProtectedResponse>("/protected");
				Assert.Fail("Should throw");
			}
			catch (WebServiceException webEx)
			{
				var response = (ProtectedResponse)webEx.ResponseDto;
				Assert.AreEqual(response.ResponseStatus.ErrorCode, typeof(UnauthorizedAccessException).Name);
				Assert.AreEqual(response.ResponseStatus.Message, "SessionId Required");
			}
		}

		[TestMethod]
		public void Can_Access_ProtectedService_After_Login()
		{
			var loginResponse = restClient.Post<LoginResponse>("/login",
				new Login { ApiKey = "MyApiKey", UserName = "NormalUser" });

			var response = restClient.Get<ProtectedResponse>("/protected?SessionId=" + loginResponse.SessionId);

			Assert.AreEqual(response.Result, HasSuccess);
		}

		[TestMethod]
		public void Can_NOT_Access_ProtectedClientType_as_NormalUser_After_Login()
		{
			var loginResponse = restClient.Post<LoginResponse>("/login",
				new Login { ApiKey = "MyApiKey", UserName = "NormalUser" });

			try
			{
				var response = restClient.Get<ProtectedClientTypeResponse>("/protectedclient?SessionId=" + loginResponse.SessionId);
				Assert.Fail("Should throw");
			}
			catch (WebServiceException webEx)
			{
				var response = (ProtectedClientTypeResponse)webEx.ResponseDto;
				Assert.AreEqual(response.ResponseStatus.ErrorCode, typeof(UnauthorizedAccessException).Name);
				Assert.AreEqual(response.ResponseStatus.Message, "Unauthorized. Requires ClientType: SuperUser");
			}
		}

		[TestMethod]
		public void Can_Access_ProtectedClientType_as_HazSuperUser_After_Login()
		{
			var loginResponse = restClient.Post<LoginResponse>("/login",
				new Login { ApiKey = "MyApiKey", UserName = "HazSuperUser" });

			var response = restClient.Get<ProtectedClientTypeResponse>("/protectedclient?SessionId=" + loginResponse.SessionId);

			Assert.AreEqual(response.Result, HasSuccess);
		}

	}
}
